#ifndef MESH_H
#define MESH_H

#include <vector>
#include <QtGui/QImage>

#include "Vector3D.h"


class Mesh 
{
	struct Triangle 
	{
		Triangle( int *vi, int *ti, int *ni) 
		  { memcpy(v,vi,3*sizeof(int)); memcpy(t,ti,3*sizeof(int)); memcpy(n,ni,3*sizeof(int)); }
		int v[3];	// indices of the vertices
		int t[3];	// indices of the texture coordinates
		int n[3];	// indices of the normals
	};

	struct MaterialNode
	{
		MaterialNode() { alpha = 1.0; textureIndex = -1; }
		MaterialNode(int texIdx) : textureIndex(texIdx) { setAmbientColor(); setDiffuseColor(); setSpecularColor(); }
		void setAmbientColor( float r = 0, float g = 0, float b = 0) { ambient[0] = r; ambient[1] = g; ambient[2] = b; }
		void setDiffuseColor( float r = 0, float g = 0, float b = 0) { diffuse[0] = r; diffuse[1] = g; diffuse[2] = b; }
		void setSpecularColor( float r = 0, float g = 0, float b = 0) { specular[0] = r; specular[1] = g; specular[2] = b; }
		char m_name[1024];
		float shininess;
		float ambient[3];
		float diffuse[3];
		float specular[3];
		float alpha;
		int textureIndex; // i-th TextureNode
	};

	struct TextureNode
	{
		TextureNode() { qImageTexture = 0; textureId = 0; texname[0] = '\0'; }
		~TextureNode() { delete qImageTexture; }
		QImage *qImageTexture;
		GLuint textureId;
		char texname[256];
	};
 
	struct MeshPartInfo
	{
		MeshPartInfo() { fromFace = toFace = materialIndex = -1; }
		MeshPartInfo(int from, int to) : fromFace(from), toFace(to), materialIndex(-1) { }
		int fromFace, toFace;
		int materialIndex;
	};

public:
	Mesh(char *filename) { readObj(filename); }  
	~Mesh() { clear(); }

	void draw();
	void loadTextureImages();
	void assignTexture(char *filename);
	bool success() const { return m_meshRead; }

	// Number of vertices and number of faces
	int getNumVertices() const { return static_cast<int>(m_vertices.size()); }
	int getNumFaces() const { return static_cast<int>(m_triangles.size()); }

	void generateSphereTexCoords();
	void generateCylinderTexCoords();

	const Vector3D& getBBoxMin() const { return m_bBoxMin; }
	const Vector3D& getBBoxMax() const { return m_bBoxMax; }

private:
	void readObj( const char* sFilename);
	void loadMaterials( const char* sFilename);
	void computeBoundingBox();
	bool loadTexture(QImage &qimg, GLuint &textureId, const char *filename);
	int getTextureNode(char *filename);
	void replaceFilename(char *tgt, const char *src, const char *newName);
	void clear();
	void correctTheta(float multp);


	bool						m_meshRead;
	bool						m_objTexAvail;

	std::vector<Vector3D>		m_vertices;
	std::vector<Vector3D>		m_normals;
	std::vector<Vector3D>		m_texCoords;
	std::vector<Triangle>		m_triangles;
	std::vector<TextureNode>	m_textures;

	std::vector<MeshPartInfo*>	m_meshPartInfo;
	std::vector<MaterialNode*>	m_materialList;

	Vector3D					m_bBoxMin;
	Vector3D					m_bBoxMax;
};

#endif // MESH_H
